import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime as dt
import sys
from KEMP import Basic_FDTD, Dielectric, to_epr, to_SI, to_NU, Gold, Silver, PA
from KEMP import structures as stc

nm = 1e-9
dx, dy, dz = [0.5*nm, 0.5*nm, 0.5*nm]
nx, ny, nz = [52, 52, 4000]
x = np.ones(nx, dtype=np.float64)*dx # 0 ~ 3000 nm
y = np.ones(ny, dtype=np.float64)*dy # 0 ~ 1 nm
z = np.ones(nz, dtype=np.float64)*dz # 0 ~ 1 nm
lx, ly, lz = dx*nx, dy*ny, dz*nz
space_grid = (x, y ,z)
#engine_name = 'nvidia_cuda'
engine_name = 'intel_cpu'
fdtd = Basic_FDTD('3D', space_grid, dtype=np.complex64, engine=engine_name, device_id=2)

pml_apply = {'x':'' , 'y':''  , 'z':'+-'  }
pbc_apply = {'x':True, 'y':True, 'z':False}
fdtd.apply_PML(pml_apply)
fdtd.apply_PBC(pbc_apply)


gap = 25*nm


diel = Dielectric(to_epr(fdtd, n=1.38))
diel2 = Dielectric(to_epr(fdtd, n=1.5))

slab=stc.Sphere(Gold,(lx/2,ly/2,lz/2),10.*nm)

#diep = Dielectric(to_epr(fdtd, n=10.**10))
slab1 = stc.Box(Gold, ((147*nm,0.*nm,gap+0.*nm),(454.*nm,ly,gap+25.*nm)))
slab2 = stc.Box(diel, ((147.*nm,0.*nm,gap+25.*nm),(454.*nm,ly,gap+60.*nm)))
slab3 = stc.Box(Gold, ((147.*nm,0.*nm,gap+60.*nm),(454.*nm,ly,gap+85.*nm)))
slab4 = stc.Box(Gold, ((0.*nm,250.*nm,gap+0.*nm),(lx,350.*nm,gap+25.*nm)))
slab5 = stc.Box(diel, ((0.*nm,250.*nm,gap+25.*nm),(lx,350.*nm,gap+60.*nm)))
slab6 = stc.Box(Gold, ((0.*nm,250.*nm,gap+60.*nm),(lx,350.*nm,gap+85.*nm)))
slab7 = stc.Box(diel2, ((0.*nm,0.*nm,gap+85.*nm),(lx,ly,gap+90.*nm)))

#
#slab = stc.Box(Gold, ((1000.*nm,0.*nm,0.*nm),(1000.*nm+slab_thick,ly,lz)))
#slab = stc.Box(PA  , ((1000.*nm,0.*nm,0.*nm),(1000.*nm+slab_thick,ly,lz)))
#pecs = stc.Box(diep, ((1000.*nm+slab_thick+gap,0.*nm,0.*nm),(lx,ly,lz)))
#slab = stc.Box(Silver, ((1000.*nm,0.*nm,0.*nm),(1000.*nm+slab_thick,ly,lz)))
#structures = [slab, pecs]
structures = [slab]

from scipy.constants import c
wavelength = np.arange(300, 901, 1.)*nm
freqs = c/wavelength
#freq0 = c/(650.*nm)
freq0 = c/(500.*nm)
wfreq0_NU = to_NU(fdtd, 'angular frequency', 2.*np.pi*freq0)



src_pos=1000
trs_pos=-1000
#rft_ey = fdtd.apply_RFT('ey', ((-50,ny/2,nz/2), (-50,ny/2,nz/2)), freqs)
inc    = fdtd.apply_direct_source('ey', ((0,0,src_pos),(-1,-1,src_pos)))
fdtd.set_structures(structures)
print 'Setting Complete and READY to RUN'

#print fdtd.cpwd.dpolenum_max, fdtd.cpwd.cpolenum_max
#
#print fdtd.cpwd.my[:,:,:]
#print fdtd.cpwd.nx, fdtd.cpwd.ny, fdtd.cpwd.nz
#print fdtd.cpwd.mx.shape, fdtd.cpwd.my.shape, fdtd.cpwd.mz.shape
#
#print fdtd.cpwd.dr_cf_r[0][1:], fdtd.cpwd.dr_ce_r[0][1:]
#print fdtd.cpwd.cp_cf_r[0][1:], fdtd.cpwd.cp_cf_i[0][1:], fdtd.cpwd.cp_ce_r[0][1:], fdtd.cpwd.cp_ce_i[0][1:]
#print fdtd.cpwd.cp_cf_r[1][1:], fdtd.cpwd.cp_cf_i[1][1:], fdtd.cpwd.cp_ce_r[1][1:], fdtd.cpwd.cp_ce_i[1][1:]

plt.ion()

t0 = dt.now()
tc = 500
ts = 50.
tmax = 15000 # 5 period for 1000 nm wavelength

ey_ref = np.zeros(tmax, dtype=np.complex64)
ey_trs = np.zeros(tmax, dtype=np.complex64)

for tstep in xrange(tmax):
    src = np.exp(-(tstep-tc)**2/(ts**2))
#    src = np.sin(wfreq0_NU*tstep*fdtd.dt)
    
    ey_ref[tstep] = fdtd.ey[:,:,src_pos].mean() - src
    ey_trs[tstep] = fdtd.ey[:,:,trs_pos].mean()
    inc.set_source(src)
    fdtd.updateE()
    fdtd.updateH()
#    print abs(fdtd.cpwd.dr_fy_r[0][:,:,:]).sum(), \
#          abs(fdtd.cpwd.cp_fy_r[0][:,:,:]).sum(), abs(fdtd.cpwd.cp_fy_i[0][:,:,:]).sum(), \
#          abs(fdtd.cpwd.cp_fy_r[1][:,:,:]).sum(), abs(fdtd.cpwd.cp_fy_i[1][:,:,:]).sum()
    if tstep % 1000 == 0:
        t1 = dt.now()
        print('%s %05d time step\r' % (t1-t0, tstep)),
        sys.stdout.flush()
        plt.clf()
        plt.plot(fdtd.ey.real[nx/2,ny/2,:], 'r')
        plt.plot(fdtd.ey.imag[nx/2,ny/2,:], 'b')
#        plt.plot(fdtd.cpwd.dr_fy_r[0][:,ny/2,nz/2], 'g')
#        plt.plot(fdtd.cpwd.cp_fy_r[0][:,ny/2,nz/2], 'b')
#        plt.plot(fdtd.cpwd.cp_fy_r[1][:,ny/2,nz/2], 'k')
        plt.ylim(-2., 2.)
        plt.draw()
#        plt.show()




tsteps = np.arange(tmax, dtype=fdtd.dtype)
ey_src = np.exp(-(tsteps-tc)**2/(ts**2))


t1 = dt.now()
print t1-t0, 'Simulation Complete'

dt_SI = to_SI(fdtd, 'time', fdtd.dt)
tsteps = np.arange(tmax, dtype=fdtd.dtype)
t = tsteps*dt_SI
ey_src = np.exp(-(tsteps-tc)**2/(ts**2))
ey_src_ft = ((dt_SI/(2.*np.pi))*ey_src[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
ey_ref_ft = ((dt_SI/(2.*np.pi))*ey_ref[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)
ey_trs_ft = ((dt_SI/(2.*np.pi))*ey_trs[np.newaxis,:]*np.exp(-2.j*np.pi*freqs[:,np.newaxis]*t[np.newaxis,:])).sum(1)



import h5py as h5
f = h5.File('./save/%s.h5' %'kemptestgold','w')
f.create_dataset('T', data=ey_trs)
f.create_dataset('I', data=ey_src)
f.create_dataset('Tf', data=ey_trs_ft)
f.create_dataset('If', data=ey_src_ft)
f.create_dataset('Rf', data=ey_ref_ft)
f.close()






'''
its_trs_ft = np.zeros_like(its_src_ft)
its_trs_arr = (abs(ey_trs_ft)**2)
for ii in xrange(its_trs_ft.size):
    its_trs_ft[ii] = its_trs_arr[:,:,ii].mean()


from RTCalculator import TransferMatrix, IsoMaterial
from RTCalculator import Gold as tmmgold, Silver as tmmsilver

def epr_inc(wavelength): return 1.
def epr_mid(wavelength): return 3.**2
def epr_tra(wavelength): return 1.

def mur_inc(wavelength): return 1.
def mur_mid(wavelength): return 1.
def mur_tra(wavelength): return 1.

inc_mat = IsoMaterial(epr_inc, mur_inc)
mid_mat = tmmgold
#mid_mat = tmmsilver
#mid_mat = IsoMaterial(epr_mid, mur_mid)
tra_mat = IsoMaterial(epr_tra, mur_tra)

tmm = TransferMatrix('E', inc_mat, tra_mat, [mid_mat], [slab_thick])
tmm.set_Wavelength(300.*nm, 1000.*nm, 1.*nm)
tmm.set_IncAngleRad(0.)
tmm.calculation(['T'])
trs_tmm = tmm.results['T'][:,0]

wave_norm = wavelength/nm

plt.clf()
plt.ioff()
plt.plot(wave_norm, its_trs_ft/its_src_ft, 'ro', label='FDTD simulation data')
plt.plot(wave_norm, trs_tmm, 'b+', label='Theoretical data')
plt.xlim(300, 1000)
plt.ylim(  0,    1)
plt.legend()
plt.xlabel('wavelength (nm)')
plt.ylabel('Transmittance')
plt.show()
'''
